@@ -11,7 +11,7 @@
  * This file contains all the "glue code" required to use the canon
  * driver with libgphoto2.
  *
- * $Id: library.c,v 1.83 2004/11/17 20:28:34 swestin Exp $
+ * $Id: library.c,v 1.93.2.1 2005/06/07 17:45:17 hfiguiere Exp $
  *
  ****************************************************************************/
 
@@ -66,14 +66,6 @@
 #include "serial.h"
 #include "usb.h"
 
-#ifndef HAVE_SNPRINTF
-# ifdef __GCC__
-#  warning You do not seem to have a snprintf() function. Using sprintf instead.
-#  warning Note that this leads to SECURITY RISKS!
-# endif
-#define snprintf(buf,size,format,arg) sprintf(buf,format,arg)
-#endif
-
 #ifndef HAVE_TM_GMTOFF
 /* required for time conversions in camera_summary() */
 extern long int timezone;
@@ -104,8 +96,7 @@ extern long int timezone;
  *  string is copied into @id.
  *
  */
-int
-camera_id (CameraText *id)
+int camera_id (CameraText *id)
 {
 	GP_DEBUG ("camera_id()");
 
@@ -130,12 +121,22 @@ camera_manual (Camera *camera, CameraText *manual, GPContext *context)
 {
 	GP_DEBUG ("camera_manual()");
 
-	strcpy (manual->text,
-		_("For the A50, using 115200 may effectively be slower than using 57600\n"
-		  "If you experience a lot of serial transmission errors,"
-		  " try to have your computer as idle as possible\n"
-		  " (i.e. no disk activity)\n"
-			));
+	strncpy (manual->text,
+		 _("This is the driver for Canon PowerShot, Digital IXUS, IXY Digital,\n"
+		   " and EOS Digital cameras in their native (sometimes called \"normal\")\n"
+		   " mode. It also supports a small number of Canon digital camcorders\n"
+		   " with still image capability.\n"
+		   "It includes code for communicating over a serial port or USB connection,\n"
+		   " but not (yet) over IEEE 1394 (Firewire).\n"
+		   "It is designed to work with over 70 models as old as the PowerShot A5\n"
+		   " and Pro70 of 1998 and as new as the PowerShot A510 and EOS 350D of\n"
+		   " 2005.\n"
+		   "It has not been verified against the EOS 1D or EOS 1Ds.\n"
+		   "For the A50, using 115200 bps may effectively be slower than using 57600\n"
+		   "If you experience a lot of serial transmission errors, try to have your\n"
+		   " computer as idle as possible (i.e. no disk activity)\n"),
+		   sizeof ( CameraText )
+		);
 
 	return GP_OK;
 }
@@ -161,7 +162,9 @@ camera_abilities (CameraAbilitiesList *list)
 	for (i = 0; models[i].id_str; i++) {
 		memset (&a, 0, sizeof (a));
 
-		if ((UPLOAD_BOOL || (models[i].usb_capture_support == CAP_EXP)) && 
+		/* For now, flag EOS 20D as experimental. */
+		if ((UPLOAD_BOOL || (models[i].usb_capture_support == CAP_EXP)
+		     || models[i].model == CANON_CLASS_6) && 
 		    (models[i].usb_vendor && models[i].usb_product)) {
 			a.status = GP_DRIVER_STATUS_EXPERIMENTAL;
 		} else {
@@ -235,17 +238,21 @@ clear_readiness (Camera *camera)
 static int
 check_readiness (Camera *camera, GPContext *context)
 {
+	int status;
+
 	GP_DEBUG ("check_readiness() cached_ready == %i", camera->pl->cached_ready);
 
 	if (camera->pl->cached_ready)
 		return 1;
-	if (canon_int_ready (camera, context) == GP_OK) {
+	status = canon_int_ready (camera, context);
+	if ( status == GP_OK ) {
 		GP_DEBUG ("Camera type: %s (%d)\n", camera->pl->md->id_str,
 			  camera->pl->md->model);
 		camera->pl->cached_ready = 1;
 		return 1;
 	}
-	gp_context_error (context, _("Camera unavailable"));
+	gp_context_error ( context, _("Camera unavailable: %s"),
+			   gp_result_as_string ( status ) );
 	return 0;
 }
 
@@ -288,11 +295,8 @@ canon_int_switch_camera_off (Camera *camera, GPContext *context)
 static int
 camera_exit (Camera *camera, GPContext *context)
 {
-	if (camera->port->type == GP_PORT_USB) {
-		/* EOS-class cameras lock keys only for remote
-		   capture, and unlock immediately. */
-		if ( !IS_EOS(camera->pl->md->model) ) canon_usb_unlock_keys (camera, context);
-	}
+	if (camera->port->type == GP_PORT_USB)
+		canon_usb_unlock_keys (camera, context);
 
 	if (camera->pl) {
 		canon_int_switch_camera_off (camera, context);
@@ -318,12 +322,13 @@ camera_exit (Camera *camera, GPContext *context)
 static int
 camera_capture_preview (Camera *camera, CameraFile *file, GPContext *context)
 {
-	int size;
+	int size, code;
 	unsigned char *data;
 
 	GP_DEBUG ("canon_capture_preview() called");
 
-	if (canon_int_capture_preview (camera, &data, &size, context) != GP_OK) {
+	code = canon_int_capture_preview (camera, &data, &size, context);
+	if ( code != GP_OK) {
 		gp_context_error (context, _("Error capturing image"));
 		return GP_ERROR;
 	}
@@ -481,14 +486,14 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 	char *filetype;
 	char buf[32];
 
-	/* put complete canon path into canon_path */
+	/* Assemble complete canon path into canon_path */
 	ret = snprintf (canon_path, sizeof (canon_path) - 3, "%s\\%s",
 			gphoto2canonpath (camera, folder, context), filename);
 	if (ret < 0) {
 		gp_context_error (context,
 				  _("Internal error #1 in get_file_func() (%s line %i)"),
 				  __FILE__, __LINE__);
-		return GP_ERROR;
+		return GP_ERROR_BAD_PARAMETERS;
 	}
 
 	/* for debugging */
@@ -531,7 +536,7 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 
 				gp_filesystem_get_info (fs, folder, filename, &info, context);
 				if (info.file.status == GP_FILE_STATUS_NOT_DOWNLOADED)
-					attr |= CANON_ATTR_DOWNLOADED;
+					attr &= ~CANON_ATTR_DOWNLOADED;
 				if ((info.file.permissions & GP_FILE_PERM_DELETE) == 0)
 					attr |= CANON_ATTR_WRITE_PROTECTED;
 				canon_int_set_file_attributes (camera, filename,
@@ -684,7 +689,10 @@ get_file_func (CameraFilesystem *fs, const char *folder, const char *filename,
 			break;
 #ifdef HAVE_EXIF
 		case GP_FILE_TYPE_EXIF:
-			gp_file_set_mime_type (file, GP_MIME_JPEG);
+			if ( !is_cr2 ( filename ) )
+				gp_file_set_mime_type (file, GP_MIME_JPEG);
+			else
+				gp_file_set_mime_type (file, GP_MIME_EXIF);
 			gp_file_set_data_and_size (file, data, datalen);
 			break;
 #endif /* HAVE_EXIF */
@@ -754,7 +762,7 @@ camera_summary (Camera *camera, CameraText *summary, GPContext *context)
 
 	GP_DEBUG ("camera_summary()");
 
-	if (check_readiness (camera, context) != 1)
+	if (!check_readiness (camera, context))
 		return GP_ERROR;
 
 	/*clear_readiness(); */
@@ -799,8 +807,8 @@ camera_summary (Camera *camera, CameraText *summary, GPContext *context)
 		  (long)tmp_time, (long)local_time, (long)tm->tm_gmtoff);
 #else
 	local_time = tmp_time - timezone;
-	GP_DEBUG ("camera_summary: converted %i to localtime %i (timezone is %i)",
-		  tmp_time, local_time, timezone);
+	GP_DEBUG ("camera_summary: converted %ld to localtime %ld (timezone is %ld)",
+		  (long)tmp_time, (long)local_time, (long)timezone);
 #endif
 
 	if (res == GP_OK) {
@@ -837,10 +845,14 @@ camera_about (Camera *camera, CameraText *about, GPContext *context)
 
 	strcpy (about->text,
 		_("Canon PowerShot series driver by\n"
-		  "Wolfgang G. Reissnegger,\n"
-		  "Werner Almesberger,\n"
-		  "Edouard Lafargue,\n"
-		  "Philippe Marzouk,\n" "A5 additions by Ole W. Saastad\n" "Holger Klemm\n")
+		  " Wolfgang G. Reissnegger,\n"
+		  " Werner Almesberger,\n"
+		  " Edouard Lafargue,\n"
+		  " Philippe Marzouk,\n"
+		  "A5 additions by Ole W. Saastad\n"
+		  "Additional enhancements by\n"
+		  " Holger Klemm\n"
+		  " Stephen H. Westin")
 		);
 
 	return GP_OK;
@@ -863,7 +875,7 @@ delete_file_func (CameraFilesystem *fs, const char *folder, const char *filename
 	strncpy (canonfolder, _canonfolder, sizeof (canonfolder) - 1);
 	canonfolder[sizeof (canonfolder) - 1] = 0;
 
-	if (check_readiness (camera, context) != 1)
+	if (!check_readiness (camera, context))
 		return GP_ERROR;
 
 	if (camera->pl->md->model == CANON_CLASS_3) {
@@ -1007,13 +1019,13 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file, void
 {
 	Camera *camera = data;
 	char destpath[300], destname[300], dir[300], dcf_root_dir[10];
-	int j, dirnum = 0, r;
+	int j, dirnum = 0, r, status;
 	char buf[10];
 	CameraAbilities a;
 
 	GP_DEBUG ("camera_folder_put_file()");
 
-	if (check_readiness (camera, context) != 1)
+	if (!check_readiness (camera, context))
 		return GP_ERROR;
 
 	gp_camera_get_abilities (camera, &a);
@@ -1066,8 +1078,9 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file, void
 		} else {
 		    char dir2[300];
 		    sprintf(dir2, "%s/%s", dcf_root_dir, dir);
-		    if (get_last_picture (camera, dir2, destname, context) == GP_ERROR)
-			    return GP_ERROR;
+		    status = get_last_picture (camera, dir2, destname, context);
+		    if ( status < 0 )
+			    return status;
 
 		    if (strlen (destname) == 0) {
 			    sprintf (destname, "AUT_%c%c01.JPG", dir[1], dir[2]);
@@ -1141,7 +1154,7 @@ put_file_func (CameraFilesystem *fs, const char *folder, CameraFile *file, void
 		return GP_ERROR_NOT_SUPPORTED;
 	}
 
-	if (check_readiness (camera, context) != 1)
+	if (!check_readiness (camera, context))
 		return GP_ERROR;
 
 	gp_camera_get_abilities (camera, &a);
@@ -1364,14 +1377,16 @@ camera_set_config (Camera *camera, CameraWidget *window, GPContext *context)
 	if (gp_widget_changed (w)) {
 		/* XXXXX mark CameraFS as dirty */
 		gp_widget_get_value (w, &camera->pl->list_all_files);
-		GP_DEBUG ("New config value for tmb: %i", camera->pl->list_all_files);
+		GP_DEBUG ("New config value for \"List all files\" %i",
+			  camera->pl->list_all_files);
 	}
-	
+
 #ifdef CANON_EXPERIMENTAL_UPLOAD
 	gp_widget_get_child_by_label (window, _("Keep filename on upload"), &w);
 	if (gp_widget_changed (w)) {
 		gp_widget_get_value (w, &camera->pl->upload_keep_filename);
-		GP_DEBUG ("New config value for tmb: %i", camera->pl->upload_keep_filename);
+		GP_DEBUG ("New config value for \"Keep filename on upload\": %i",
+			  camera->pl->upload_keep_filename);
 	}
 #endif /* CANON_EXPERIMENTAL_UPLOAD */
 
@@ -1442,7 +1457,7 @@ make_dir_func (CameraFilesystem *fs, const char *folder, const char *name, void
 
 	canonpath = gphoto2canonpath (camera, gppath, context);
 	if (canonpath == NULL)
-		return GP_ERROR;
+		return GP_ERROR_BAD_PARAMETERS;
 
 	r = canon_int_directory_operations (camera, canonpath, DIR_CREATE, context);
 	if (r != GP_OK)
@@ -1482,7 +1497,7 @@ remove_dir_func (CameraFilesystem *fs, const char *folder, const char *name, voi
 
 	canonpath = gphoto2canonpath (camera, gppath, context);
 	if (canonpath == NULL)
-		return GP_ERROR;
+		return GP_ERROR_BAD_PARAMETERS;
 
 	r = canon_int_directory_operations (camera, canonpath, DIR_REMOVE, context);
 	if (r != GP_OK)
