@@ -2,7 +2,7 @@
  *
  * File: util.h
  *
- * $Id: util.h,v 1.16 2003/02/27 14:52:08 swestin Exp $
+ * $Id: util.h,v 1.17.2.1 2005/06/07 17:46:10 hfiguiere Exp $
  ****************************************************************************/
 
 #ifndef _CANON_UTIL_H
@@ -22,8 +22,7 @@
  * prototypes
  *
  ****************************************************************************/
-
-/*void dump_hex(Camera *camera,const char *msg, const unsigned char *buf, int len);*/
+void dump_hex ( FILE *fp, void *buffer, int length );
 
 int is_thumbnail (const char *name);
 int is_audio (const char *name);
@@ -31,6 +30,7 @@ int is_image (const char *name);
 int is_movie (const char *name);
 int is_jpeg (const char *name);
 int is_crw (const char *name);
+int is_cr2 (const char *name);
 
 const char *filename2mimetype(const char *filename);
 
