@@ -1,5 +1,5 @@
 /*
-	$Id: dc.h,v 1.14 2002/12/15 11:43:27 asl Exp $
+	$Id: dc.h,v 1.15 2005/04/05 21:04:19 hun Exp $
 
 	Copyright  2000 Mariusz Zynel <mariusz@mizar.org> (gPhoto port)
 	Copyright  2000 Fredrik Roubert <roubert@df.lth.se> (idea)
@@ -60,13 +60,6 @@ struct _CameraPrivateLibrary {
 #define EDSCOVERFL	5 /* overfolw */
 #define EDSCMAXERR	5 /* highest used error code */
 
-static const char
-	c_prefix[13] = /* generic command prefix */
-	{ 'M', 'K', 'E', ' ', 'D', 'S', 'C', ' ', 'P', 'C', ' ', ' ' },
-
-	r_prefix[13] = /* generic response prefix */
-	{ 'M', 'K', 'E', ' ', 'P', 'C', ' ', ' ', 'D', 'S', 'C', ' ' };
-
 int dsc1_sendcmd(Camera *camera, uint8_t cmd, void *data, int size);
 	/* Send command with 'data' of 'size' to DSC */
 
