@@ -138,16 +138,44 @@ typedef struct _PTPUSBEventContainer PTPUSBEventContainer;
 #define PTP_OC_CopyObject               0x101A
 #define PTP_OC_GetPartialObject         0x101B
 #define PTP_OC_InitiateOpenCapture      0x101C
+
 /* Eastman Kodak extension Operation Codes */
 #define PTP_OC_EK_SendFileObjectInfo	0x9005
 #define PTP_OC_EK_SendFileObject	0x9006
+
 /* Canon extension Operation Codes */
 #define PTP_OC_CANON_GetObjectSize	0x9001
+#define PTP_OC_CANON_9002		0x9002
+/* 9002 - sends 2 uint32, nothing back  */
+#define PTP_OC_CANON_9003		0x9003
+/* 9003 - sends nothing, nothing back  */
+/* no 9004 observed yet */
+/* no 9005 observed yet */
+/* 9006 - sends a ptp string, gets back a uint32 */
+#define PTP_OC_CANON_9006		0x9006
+/* no 9007 observed yet */
 #define PTP_OC_CANON_StartShootingMode	0x9008
 #define PTP_OC_CANON_EndShootingMode	0x9009
+/* 900a - sends nothing, nothing back */
+#define PTP_OC_CANON_900a		0x900A
 #define PTP_OC_CANON_ViewfinderOn	0x900B
 #define PTP_OC_CANON_ViewfinderOff	0x900C
 #define PTP_OC_CANON_ReflectChanges	0x900D
+
+/* 900e - send nothing, gets 5 uint16t in 32bit entities back in 20byte datablob */
+#define PTP_OC_CANON_900e		0x900E
+/* 900f - gets data ... hmm, object handles ? */
+#define PTP_OC_CANON_900f		0x900F
+
+#define PTP_OC_CANON_ThemeDownload	0x9010
+#define PTP_OC_CANON_ThemeUpload	0x9011
+
+
+/* initiate movie capture:
+   9010 startmoviecapture?
+   9003 stopmoviecapture?
+*/
+
 #define PTP_OC_CANON_CheckEvent		0x9013
 #define PTP_OC_CANON_FocusLock		0x9014
 #define PTP_OC_CANON_FocusUnlock	0x9015
@@ -222,9 +250,9 @@ typedef struct _PTPUSBEventContainer PTPUSBEventContainer;
 #define PTP_EC_CaptureComplete		0x400D
 #define PTP_EC_UnreportedStatus		0x400E
 /* Canon extension Event Codes */
-#define PTP_EC_CANON_DeviceInfoChanged	0xC008
+#define PTP_EC_CANON_ObjectInfoChanged		0xC008
 #define PTP_EC_CANON_RequestObjectTransfer	0xC009
-#define PTP_EC_CANON_CameraModeChanged	0xC00C
+#define PTP_EC_CANON_CameraModeChanged		0xC00C
 
 /* PTP device info structure (returned by GetDevInfo) */
 
@@ -631,16 +659,36 @@ typedef struct _PTPCANONFolderEntry PTPCANONFolderEntry;
 #define PTP_DPC_NIKON_MenusAndPlayback			0xD088
 #define PTP_DPC_NIKON_F6ButtonsAndDials			0xD089
 #define PTP_DPC_NIKON_F7NoCFCard			0xD08a
+#define PTP_DPC_NIKON_ImageCommentString		0xD090
 #define PTP_DPC_NIKON_AutoImageRotation			0xD092
 #define PTP_DPC_NIKON_ExposureBracketingOnOff		0xD0c0
 #define PTP_DPC_NIKON_ExposureBracketingIntervalDist	0xD0c1
 #define PTP_DPC_NIKON_ExposureBracketingNumBracketPlace	0xD0c2
+#define PTP_DPC_NIKON_LensID				0xD0e0
+#define PTP_DPC_NIKON_FocalLengthMin			0xD0e3
+#define PTP_DPC_NIKON_FocalLengthMax			0xD0e4
+#define PTP_DPC_NIKON_MaxApAtMinFocalLength		0xD0e5
+#define PTP_DPC_NIKON_MaxApAtMaxFocalLength		0xD0e6
 #define PTP_DPC_NIKON_AutofocusLCDTopMode2		0xD107
 #define PTP_DPC_NIKON_AutofocusLCDTopMode3AndMode4	0xD108
 #define PTP_DPC_NIKON_LightMeter			0xD10a
 #define PTP_DPC_NIKON_ExposureAperatureLock(ReadOnly)	0xD111
 #define PTP_DPC_NIKON_MaximumShots			0xD103
-
+#define PTP_DPC_NIKON_FlashExposureCompensation		0xD126
+#define PTP_DPC_NIKON_Beep				0xD160
+#define PTP_DPC_NIKON_AFC				0xD161
+#define PTP_DPC_NIKON_AFLampOff				0xD163
+#define PTP_DPC_NIKON_PADVPMode				0xD164
+#define PTP_DPC_NIKON_ReviewOff				0xD165
+#define PTP_DPC_NIKON_GridDisplay			0xD16c
+#define PTP_DPC_NIKON_AFAreaIllumination		0xD166
+#define PTP_DPC_NIKON_FlashMode				0xD167
+#define PTP_DPC_NIKON_FlashPower			0xD16d
+#define PTP_DPC_NIKON_FlashSignOff			0xD169
+#define PTP_DPC_NIKON_RemoteTimeout			0xD16B
+#define PTP_DPC_NIKON_LowLight				0xD1b0
+#define PTP_DPC_NIKON_FlashOpen				0xD1c0
+#define PTP_DPC_NIKON_FlashCharged			0xD1c1
 
 /* Device Property Form Flag */
 
@@ -748,6 +796,8 @@ uint16_t ptp_getobjectinfo	(PTPParams *params, uint32_t handle,
 
 uint16_t ptp_getobject		(PTPParams *params, uint32_t handle,
 				char** object);
+uint16_t ptp_getpartialobject	(PTPParams* params, uint32_t handle, uint32_t offset,
+				uint32_t maxbytes, char** object);
 uint16_t ptp_getthumb		(PTPParams *params, uint32_t handle,
 				char** object);
 
@@ -807,10 +857,12 @@ uint16_t ptp_canon_getfolderentries (PTPParams* params, uint32_t store,
 				uint32_t handle, 
 				PTPCANONFolderEntry** entries,
 				uint32_t* entnum);
-
+uint16_t ptp_canon_theme_download (PTPParams* params, uint32_t themenr,
+				char **data, unsigned int *size);
 
 /* Non PTP protocol functions */
 int ptp_operation_issupported	(PTPParams* params, uint16_t operation);
+int ptp_event_issupported	(PTPParams* params, uint16_t event);
 int ptp_property_issupported	(PTPParams* params, uint16_t property);
 
 void ptp_free_devicepropdesc	(PTPDevicePropDesc* dpd);
@@ -820,4 +872,8 @@ void ptp_perror			(PTPParams* params, uint16_t error);
 const char*
 ptp_get_property_description(PTPParams* params, uint16_t dpc);
 
+int
+ptp_render_property_value(PTPParams* params, uint16_t dpc,
+                          PTPDevicePropDesc *dpd, int length, char *out);
+
 #endif /* __PTP_H__ */
