@@ -250,6 +250,26 @@ camera_abilities (CameraAbilitiesList *list)
 	a.usb_product	= 0x220b;
 	gp_abilities_list_append(list, a);
 
+	strcpy(a.model, "Ricoh:Caplio R1");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x220d;
+	gp_abilities_list_append(list, a);
+
+	/* same id as above */
+	strcpy(a.model, "Ricoh:Caplio RZ1");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x220d;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Sea & Sea:5000G");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x220e;
+	gp_abilities_list_append(list, a);
+
+	strcpy(a.model, "Ricoh:Caplio R1v");
+	a.usb_vendor	= 0x5ca;
+	a.usb_product	= 0x2212;
+	gp_abilities_list_append(list, a);
 
 	strcpy(a.model, "Medion:MD 6126");
 	a.usb_vendor    = 0x5ca;
@@ -746,9 +766,9 @@ file_list_func (CameraFilesystem *fs, const char *folder, CameraList *list,
 				char xfn[13];
 				int date, time, month, year;
 
-				strcpy(xfn, buf+n*32);
+				strncpy (xfn, buf+n*32, 8);
 				xfn[8] = '.';
-				strcpy(xfn+9, buf+n*32+8);
+				strncpy (xfn+9, buf+n*32+8, 3);
 				xfn[8+1+3] = '\0';
 
 				ret = gp_filesystem_append (fs, folder, xfn, context);
