@@ -85,7 +85,7 @@ static struct {
 	{"Epson", "PhotoPC 550", SIERRA_MODEL_EPSON,	0, 0, 0 },
 	{"Epson", "PhotoPC 600", SIERRA_MODEL_EPSON,	0, 0, 0 },
 	{"Epson", "PhotoPC 650", SIERRA_MODEL_EPSON,	0, 0,
-					SIERRA_NO_51 },
+					SIERRA_NO_51 | SIERRA_LOW_SPEED },
 	{"Epson", "PhotoPC 700", SIERRA_MODEL_EPSON,	0, 0, 0 },
 	{"Epson", "PhotoPC 800", SIERRA_MODEL_EPSON,	0, 0, 0 },
 	{"Epson", "PhotoPC 850z", SIERRA_MODEL_EPSON,     0x4b8, 0x402, 0},
@@ -103,8 +103,17 @@ static struct {
 	{"Nikon", "CoolPix 950S", SIERRA_MODEL_DEFAULT,	0, 0, 0 },
 	{"Nikon", "CoolPix 990", SIERRA_MODEL_DEFAULT,	0x04b0, 0x0102, 0},
 	{"Nikon", "CoolPix 995",   SIERRA_MODEL_CAM_DESC, 0x04b0, 0x0104,
-						SIERRA_WRAP_USB,
+						SIERRA_WRAP_USB_OLYMPUS,
 							&cp995_cam_desc},
+	{"Nikon", "CoolPix 2500 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x0108,
+						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
+							&cp880_cam_desc},
+	{"Nikon", "CoolPix 4300 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x010e,
+						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
+							&cp880_cam_desc},
+	{"Nikon", "CoolPix 3500 (Sierra Mode)", SIERRA_MODEL_CAM_DESC, 0x04b0, 0x0110,
+						SIERRA_WRAP_USB_NIKON | SIERRA_NO_51,
+							&cp880_cam_desc},
 	{"Olympus", "D-100Z", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "D-200L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "D-220L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
@@ -126,15 +135,16 @@ static struct {
 	{"Olympus", "C-410", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-410L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-420", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus", "C-420L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-420L", 	SIERRA_MODEL_OLYMPUS,	0, 0, SIERRA_MID_SPEED },
 	{"Olympus", "C-700UZ",	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
-					SIERRA_WRAP_USB, &oly3040_cam_desc},
+					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Olympus", "C-750UZ",	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
-					SIERRA_WRAP_USB, &oly750uz_cam_desc},
+					SIERRA_WRAP_USB_OLYMPUS, &oly750uz_cam_desc},
 	{"Olympus", "C-800", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-800L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-820", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
-	{"Olympus", "C-820L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
+	{"Olympus", "C-820L", 	SIERRA_MODEL_OLYMPUS,	0, 0,
+					SIERRA_MID_SPEED },
 	{"Olympus", "C-830L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-840L", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-860L",	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
@@ -146,7 +156,7 @@ static struct {
 	{"Olympus", "C-2000Z", 	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-2020Z",	SIERRA_MODEL_OLYMPUS,	0, 0, 0 },
 	{"Olympus", "C-2040Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
-					SIERRA_WRAP_USB, &oly3040_cam_desc},
+					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Olympus", "C-2100UZ", SIERRA_MODEL_CAM_DESC, 0x07b4, 0x100, 0,
 					&oly3040_cam_desc},
 	{"Olympus", "C-2500L",     SIERRA_MODEL_OLYMPUS,   0, 0, 0 },
@@ -159,16 +169,20 @@ static struct {
 	{"Olympus", "C-3000Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x100, 0,
 							&oly3000z_cam_desc},
 	{"Olympus", "C-3020Z", SIERRA_MODEL_CAM_DESC,  0x07b4, 0x105,
-					SIERRA_WRAP_USB, &oly3040_cam_desc},
+					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Olympus", "C-3030Z", SIERRA_MODEL_CAM_DESC,	0x07b4, 0x100, 0,
 							&oly3040_cam_desc},
 	{"Olympus", "C-3040Z",     SIERRA_MODEL_CAM_DESC,  0x07b4, 0x105,
-					SIERRA_WRAP_USB, &oly3040_cam_desc},
+					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Olympus", "C-4040Z",     SIERRA_MODEL_CAM_DESC,  0x07b4, 0x105,
-					SIERRA_WRAP_USB, &oly3040_cam_desc},
+					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Olympus", "C-5050Z", 	SIERRA_MODEL_CAM_DESC,	0x07b4, 0x105,
-					SIERRA_WRAP_USB, &oly3040_cam_desc},
+					SIERRA_WRAP_USB_OLYMPUS, &oly3040_cam_desc},
 	{"Panasonic", "Coolshot NV-DCF5E", SIERRA_MODEL_DEFAULT, 0, 0, 0 },
+
+	{"Pentax", "Optio 450", SIERRA_MODEL_DEFAULT, 0x0a17,0x0007, SIERRA_WRAP_USB_PENTAX },
+	{"Pentax", "Optio 33WR", SIERRA_MODEL_DEFAULT, 0x0a17,0x0009, SIERRA_WRAP_USB_PENTAX },
+
 	{"Polaroid", "PDC 640", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
 	{"Polaroid", "PDC 2300Z", SIERRA_MODEL_DEFAULT, 0x0546, 0x0daf,
 					SIERRA_SKIP_INIT },
@@ -177,7 +191,7 @@ static struct {
 	{"Sanyo", "VPC-G200", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
 	{"Sanyo", "VPC-G200EX", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
 	{"Sanyo", "VPC-G210", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
-	{"Sanyo", "VPC-G250", 	SIERRA_MODEL_DEFAULT,	0, 0, 0 },
+	{"Sanyo", "VPC-G250", 	SIERRA_MODEL_DEFAULT,	0, 0, SIERRA_NO_51 },
 	{"Sierra Imaging", "SD640",SIERRA_MODEL_DEFAULT,	0, 0, 0 },
 	/*
 	 * Added by Sean Bruno on July 22, 2004 to support the Toshiba
@@ -229,12 +243,22 @@ int camera_abilities (CameraAbilitiesList *list)
 		a.speed[0] = 9600;
 		a.speed[1] = 19200;
 		a.speed[2] = 38400;
-		if (!(sierra_cameras[x].flags & SIERRA_LOW_SPEED)) {
-			a.speed[3] = 57600;
-			a.speed[4] = 115200;
-			a.speed[5] = 0;
-		} else {
+		if (sierra_cameras[x].flags & SIERRA_LOW_SPEED) {
+			/*
+			 * Camera supports 9600 -> 38400 
+			 */
 			a.speed[3] = 0;
+		} else {
+			a.speed[3] = 57600;
+			if (sierra_cameras[x].flags & SIERRA_MID_SPEED) {
+				/*
+				 * Camera supports 9600 -> 57600.
+				 */
+				a.speed[4] = 0;
+			} else {
+				a.speed[4] = 115200;
+				a.speed[5] = 0;
+			}
 		}
 		a.operations        = 	GP_OPERATION_CAPTURE_IMAGE |
 					GP_OPERATION_CAPTURE_PREVIEW |
@@ -1965,7 +1989,7 @@ camera_about (Camera *camera, CameraText *about, GPContext *context)
 		_("sierra SPARClite library\n"
 		"Scott Fritzinger <scottf@unr.edu>\n"
 		"Support for sierra-based digital cameras\n"
-		"including Olympus, Nikon, Epson, and others.\n"
+		"including Olympus, Nikon, Epson, Pentax, and others.\n"
 		"\n"
 		"Thanks to Data Engines (www.dataengines.com)\n"
 		"for the use of their Olympus C-3030Z for USB\n"
