@@ -102,13 +102,27 @@ typedef struct {
 static int gp_filesystem_get_file_impl (CameraFilesystem *, const char *,
 		const char *, CameraFileType, CameraFile *, GPContext *);
 
+static time_t
+get_time_from_exif_tag(ExifEntry *e) {
+	struct tm ts;
+
+        e->data[4] = e->data[ 7] = e->data[10] = e->data[13] = e->data[16] = 0;
+        ts.tm_year = atoi (e->data) - 1900;
+        ts.tm_mon  = atoi (e->data + 5) - 1;
+        ts.tm_mday = atoi (e->data + 8);
+        ts.tm_hour = atoi (e->data + 11);
+        ts.tm_min  = atoi (e->data + 14);
+        ts.tm_sec  = atoi (e->data + 17);
+
+        return mktime (&ts);
+}
+
 static time_t
 get_exif_mtime (const unsigned char *data, unsigned long size)
 {
 	ExifData *ed;
 	ExifEntry *e;
-	struct tm ts;
-	time_t t;
+	time_t t, t1 = 0, t2 = 0, t3 = 0;
 
         ed = exif_data_new_from_data (data, size);
         if (!ed) {
@@ -121,39 +135,50 @@ get_exif_mtime (const unsigned char *data, unsigned long size)
          */
 #ifdef HAVE_EXIF_0_5_4
 	e = exif_content_get_entry (ed->ifd[EXIF_IFD_0], EXIF_TAG_DATE_TIME);
-	if (!e)
-		e = exif_content_get_entry (ed->ifd[EXIF_IFD_EXIF],
-					    EXIF_TAG_DATE_TIME_ORIGINAL);
-	if (!e)
-		e = exif_content_get_entry (ed->ifd[EXIF_IFD_EXIF],
-					    EXIF_TAG_DATE_TIME_DIGITIZED);
+	if (e)
+		t1 = get_time_from_exif_tag(e);
+	e = exif_content_get_entry (ed->ifd[EXIF_IFD_EXIF],
+				    EXIF_TAG_DATE_TIME_ORIGINAL);
+	if (e)
+		t2 = get_time_from_exif_tag(e);
+	e = exif_content_get_entry (ed->ifd[EXIF_IFD_EXIF],
+				    EXIF_TAG_DATE_TIME_DIGITIZED);
+	if (e)
+		t3 = get_time_from_exif_tag(e);
 #else
 	e = exif_content_get_entry (ed->ifd0, EXIF_TAG_DATE_TIME);
-	if (!e)
-	        e = exif_content_get_entry (ed->ifd_exif,
-					    EXIF_TAG_DATE_TIME_ORIGINAL);
-	if (!e)
-		e = exif_content_get_entry (ed->ifd_exif, 
-					    EXIF_TAG_DATE_TIME_DIGITIZED);
+	if (e) {
+		t1 = get_time_from_exif_tag(e);
+		exif_data_unref (e);
+	}
+	e = exif_content_get_entry (ed->ifd_exif,
+				    EXIF_TAG_DATE_TIME_ORIGINAL);
+	if (e) {
+		t2 = get_time_from_exif_tag(e);
+		exif_data_unref (e);
+	}
+	e = exif_content_get_entry (ed->ifd_exif, 
+				    EXIF_TAG_DATE_TIME_DIGITIZED);
+	if (e) {
+		t3 = get_time_from_exif_tag(e);
+		exif_data_unref (e);
+	}
 #endif
-        if (!e) {
-                GP_DEBUG ("EXIF data has not date/time tag.");
-                exif_data_unref (ed);
+        exif_data_unref (ed);
+        if (!t1 && !t2 && !t3) {
+                GP_DEBUG ("EXIF data has not date/time tags.");
                 return 0;
         }
 
-        e->data[4] = e->data[ 7] = e->data[10] = e->data[13] = e->data[16] = 0;
-        ts.tm_year = atoi (e->data) - 1900;
-        ts.tm_mon  = atoi (e->data + 5) - 1;
-        ts.tm_mday = atoi (e->data + 8);
-        ts.tm_hour = atoi (e->data + 11);
-        ts.tm_min  = atoi (e->data + 14);
-        ts.tm_sec  = atoi (e->data + 17);
-        exif_data_unref (ed);
+	/* Perform some sanity checking on those tags */
+	t = t1; /* "last modified" */
+
+	if (t2 > t)	/* "image taken" > "last modified" ? can not be */
+		t = t2;
+	if (t3 > t)	/* "image digitized" > max(last two) ? can not be */
+		t = t3;
 
-        t = mktime (&ts);
         GP_DEBUG ("Found time in EXIF data: '%s'.", asctime (localtime (&t)));
-
         return (t);
 }
 
@@ -261,6 +286,17 @@ struct _CameraFilesystem {
 	}								\
 }
 
+/** check for buffer overflow */
+#define CBO(bufsize, string_len, msg) \
+	if (bufsize <= string_len) { \
+		GP_DEBUG ("%s: strlen(...) = %d " \
+			">= sizeof(buffer) = %d", \
+			msg, string_len, bufsize \
+		); \
+		gp_context_error (context, "preventing buffer overflow"); \
+		return GP_ERROR; \
+	}
+
 static int
 delete_all_files (CameraFilesystem *fs, int x)
 {
@@ -362,14 +398,15 @@ append_folder (CameraFilesystem *fs, const char *folder, GPContext *context)
 {
         CameraFilesystemFolder *new;
         int x;
-        char buf[128];
+        char *buf = NULL;
 
 	gp_log (GP_LOG_DEBUG, "gphoto2-filesystem",
 		"Internally appending folder %s...", folder);
 
-        CHECK_NULL (fs && folder);
+	CHECK_NULL (fs);
+	CHECK_NULL (folder);	
 	CC (context);
-        CA (folder, context);
+	CA (folder, context);
 
         /* Make sure the directory doesn't exist */
 	for (x = 0; x < fs->count; x++)
@@ -383,7 +420,11 @@ append_folder (CameraFilesystem *fs, const char *folder, GPContext *context)
         }
 
         /* Make sure the parent exist. If not, create it. */
-	strncpy (buf, folder, sizeof (buf));
+	buf = strdup (folder);
+	CHECK_NULL (buf);
+	/* Do not forget to free buf before returning in the normal case.
+	 * In case of error abortions, it probably will not matter much, though.
+	 */
         for (x = strlen (buf) - 1; x >= 0; x--)
                 if (buf[x] == '/')
                         break;
@@ -406,7 +447,9 @@ append_folder (CameraFilesystem *fs, const char *folder, GPContext *context)
         fs->count++;
 
 	/* Initialize the folder (and remove trailing slashes if necessary). */
-        strcpy (fs->folder[fs->count - 1].name, folder);
+	CBO(sizeof(fs->folder[fs->count - 1].name), strlen(folder),
+	    "append_folder(): folder >= sizeof(CameraFilesystemFolder.name)");
+	strcpy (fs->folder[fs->count - 1].name, folder);
         if ((strlen (folder) > 1) &&
             (fs->folder[fs->count - 1].name[strlen (folder) - 1] == '/'))
                 fs->folder[fs->count - 1].name[strlen (folder) - 1] = '\0';
@@ -414,11 +457,12 @@ append_folder (CameraFilesystem *fs, const char *folder, GPContext *context)
         fs->folder[fs->count - 1].files_dirty = 1;
         fs->folder[fs->count - 1].folders_dirty = 1;
 
+	free(buf);
         return (GP_OK);
 }
 
 static int
-append_file (CameraFilesystem *fs, int x, CameraFile *file)
+append_file (CameraFilesystem *fs, int x, CameraFile *file, GPContext *context)
 {
 	CameraFilesystemFile *new;
 	const char *name;
@@ -437,6 +481,9 @@ append_file (CameraFilesystem *fs, int x, CameraFile *file)
 	fs->folder[x].count++;
 	memset (&(fs->folder[x].file[fs->folder[x].count - 1]), 0,
 		sizeof (CameraFilesystemFile));
+	CBO(strlen(name), 
+	    sizeof(fs->folder[x].file[fs->folder[x].count - 1].name),
+	    "append_file()");
 	strcpy (fs->folder[x].file[fs->folder[x].count - 1].name, name);
 	fs->folder[x].file[fs->folder[x].count - 1].info_dirty = 1;
 	fs->folder[x].file[fs->folder[x].count - 1].normal = file;
@@ -1169,7 +1216,7 @@ gp_filesystem_put_file (CameraFilesystem *fs, const char *folder,
 
 	/* Upload the file */
 	CR (fs->put_file_func (fs, folder, file, fs->folder_data, context));
-	CR (append_file (fs, x, file));
+	CR (append_file (fs, x, file, context));
 
 	return (GP_OK);
 }
