@@ -172,18 +172,21 @@ foreach_func (const char *filename, lt_ptr data)
 	unsigned int j, old_size = list->count;
 	int result;
 
+	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
+		_("Called for filename '%s'."), filename );
+
 	lh = lt_dlopenext (filename);
 	if (!lh) {
-		gp_log (GP_LOG_DEBUG, "gp-port-info-list",
-			"Could not load '%s': '%s'.", filename, lt_dlerror ());
+		gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
+			_("Could not load '%s': '%s'."), filename, lt_dlerror ());
 		return (0);
 	}
 
 	lib_type = lt_dlsym (lh, "gp_port_library_type");
 	lib_list = lt_dlsym (lh, "gp_port_library_list");
 	if (!lib_type || !lib_list) {
-		gp_log (GP_LOG_DEBUG, "gp-port-info-list",
-			"Could not find some functions in '%s': '%s'.",
+		gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
+			_("Could not find some functions in '%s': '%s'."),
 			filename, lt_dlerror ());
 		lt_dlclose (lh);
 		return (0);
@@ -194,8 +197,8 @@ foreach_func (const char *filename, lt_ptr data)
 		if (list->info[j].type == type)
 			break;
 	if (j != list->count) {
-		gp_log (GP_LOG_DEBUG, "gp-port-info-list",
-			"'%s' already loaded", filename);
+		gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
+			_("'%s' already loaded"), filename);
 		lt_dlclose (lh);
 		return (0);
 	}
@@ -203,15 +206,15 @@ foreach_func (const char *filename, lt_ptr data)
 	result = lib_list (list);
 	lt_dlclose (lh);
 	if (result < 0) {
-		gp_log (GP_LOG_DEBUG, "gp-port-info-list",
-			"Could not load list: '%s'.",
+		gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
+			_("Could not load port driver list: '%s'."),
 			gp_port_result_as_string (result));
 		return (0);
 	}
 
 	for (j = old_size; j < list->count; j++){
-		gp_log (GP_LOG_DEBUG, "gp-port-info-list",
-			"Loaded '%s' ('%s') from '%s'.",
+		gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
+			_("Loaded '%s' ('%s') from '%s'."),
 			list->info[j].name, list->info[j].path,
 			filename);
 		strcpy (list->info[j].library_filename, filename);
@@ -250,10 +253,10 @@ gp_port_info_list_load (GPPortInfoList *list)
 #endif
 
 	CHECK_NULL (list);
-	gp_log (GP_LOG_DEBUG, "gp-port-info-list", "Loading io-drivers "
-		"from '" IOLIBS "'...");
 
 #ifdef HAVE_LTDL
+	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list", _("Using ltdl to load io-drivers "
+		"from '%s'..."),IOLIBS);
 	lt_dlinit ();
 	lt_dladdsearchdir (IOLIBS);
 	result = lt_dlforeachfile (IOLIBS, foreach_func, list);
@@ -262,9 +265,11 @@ gp_port_info_list_load (GPPortInfoList *list)
 		return (result);
 
 #else
+	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list", _("Loading io-drivers "
+		"from '%s' without ltdl..."),IOLIBS);
 	d = GP_SYSTEM_OPENDIR (IOLIBS);
         if (!d) {
-                gp_log (GP_LOG_ERROR, "gphoto2-port-core",
+                gp_log (GP_LOG_ERROR, "gphoto2-port-info-list",
                         _("Could not load any io-library because '%s' could "
                         "not be opened (%m)"), IOLIBS);
                 return (GP_ERROR_LIBRARY);
@@ -296,8 +301,8 @@ gp_port_info_list_load (GPPortInfoList *list)
 				    )) {
 				/* *.la or *.a - we cannot load these, so no error msg */
 			} else {
-                                gp_log (GP_LOG_DEBUG, "gphoto2-port-core",
-                                        "'%s' is not a library (%s)", path,
+                                gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
+                                        _("'%s' is not a library (%s)"), path,
                                         GP_SYSTEM_DLERROR ());
 			}
                         continue;
@@ -306,8 +311,8 @@ gp_port_info_list_load (GPPortInfoList *list)
                 lib_type = GP_SYSTEM_DLSYM (lh, "gp_port_library_type");
                 lib_list = GP_SYSTEM_DLSYM (lh, "gp_port_library_list");
                 if (!lib_type || !lib_list) {
-                        gp_log (GP_LOG_DEBUG, "gphoto2-port-core",
-                                "Could not find some functions in '%s' (%s)",
+                        gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
+                                _("Could not find some functions in '%s' (%s)"),
                                 path, GP_SYSTEM_DLERROR ());
                         GP_SYSTEM_DLCLOSE (lh);
                         continue;
@@ -318,24 +323,24 @@ gp_port_info_list_load (GPPortInfoList *list)
                         if (list->info[i].type == type)
                                 break;
                 if (i != list->count) {
-                        gp_log (GP_LOG_DEBUG, "gphoto2-port-core",
-                                "'%s' already loaded", path);
+                        gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
+                                _("'%s' already loaded"), path);
                         GP_SYSTEM_DLCLOSE (lh);
                         continue;
                 }
 
                 result = lib_list (list);
                 if (result < 0) {
-                        gp_log (GP_LOG_DEBUG, "gphoto2-port-core",
-                                "Could not load list (%s)",
+                        gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
+                                _("Could not load list (%s)"),
                                 gp_port_result_as_string (result));
                         GP_SYSTEM_DLCLOSE (lh);
                         continue;
                 }
 
                 for (i = old_size; i < list->count; i++){
-                        gp_log (GP_LOG_DEBUG, "gphoto2-port-core",
-                                "Loaded '%s' (%s) from '%s'",
+                        gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
+                                _("Loaded '%s' (%s) from '%s'"),
                                 list->info[i].name, list->info[i].path,
                                 filename);
                         strcpy (list->info[i].library_filename, path);
@@ -366,8 +371,8 @@ gp_port_info_list_count (GPPortInfoList *list)
 
 	CHECK_NULL (list);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list", "Counting entries "
-		"(%i available)...", list->count);
+	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list", _("Counting entries "
+		"(%i available)..."), list->count);
 
 	/* Ignore generic entries */
 	count = list->count;
@@ -375,8 +380,8 @@ gp_port_info_list_count (GPPortInfoList *list)
 		if (!strlen (list->info[i].name))
 			count--;
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list", "%i regular entries "
-		"available.", count);
+	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list", _("%i regular entries "
+		"available."), count);
 	return (count);
 }
 
@@ -404,8 +409,8 @@ gp_port_info_list_lookup_path (GPPortInfoList *list, const char *path)
 
 	CHECK_NULL (list && path);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list", "Looking for "
-		"path '%s' (%i entries available)...", path, list->count);
+	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list", _("Looking for "
+		"path '%s' (%i entries available)..."), path, list->count);
 
 	/* Exact match? */
 	for (generic = i = 0; i < list->count; i++)
@@ -416,13 +421,13 @@ gp_port_info_list_lookup_path (GPPortInfoList *list, const char *path)
 
 	/* Regex match? */
 	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-		"Starting regex search for '%s'...", path);
+		_("Starting regex search for '%s'..."), path);
 	for (i = 0; i < list->count; i++) {
 		if (strlen (list->info[i].name))
 			continue;
 
 		gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-			"Trying '%s'...", list->info[i].path);
+			_("Trying '%s'..."), list->info[i].path);
 
 		/* Compile the pattern */
 #ifdef HAVE_GNU_REGEX
@@ -443,7 +448,7 @@ gp_port_info_list_lookup_path (GPPortInfoList *list, const char *path)
 					"%s", buf);
 			else
 				gp_log (GP_LOG_ERROR, "gphoto2-port-info-list",
-					"regcomp failed");
+					_("regcomp failed"));
 			return (GP_ERROR_UNKNOWN_PORT);
 		}
 #endif
@@ -454,7 +459,7 @@ gp_port_info_list_lookup_path (GPPortInfoList *list, const char *path)
 		regfree (&pattern);
 		if (result < 0) {
 			gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-				"re_match failed (%i)", result);
+				_("re_match failed (%i)"), result);
 			continue;
 		}
 #else
@@ -462,7 +467,7 @@ gp_port_info_list_lookup_path (GPPortInfoList *list, const char *path)
 		regfree (&pattern);
 		if (result) {
 			gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list",
-				"regexec failed");
+				_("regexec failed"));
 			continue;
 		}
 #endif
@@ -493,8 +498,8 @@ gp_port_info_list_lookup_name (GPPortInfoList *list, const char *name)
 
 	CHECK_NULL (list && name);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list", "Looking up entry "
-		"'%s'...", name);
+	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list", _("Looking up entry "
+		"'%s'..."), name);
 
 	/* Ignore generic entries */
 	for (generic = i = 0; i < list->count; i++)
@@ -523,8 +528,8 @@ gp_port_info_list_get_info (GPPortInfoList *list, int n, GPPortInfo *info)
 
 	CHECK_NULL (list && info);
 
-	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list", "Getting info of "
-		"entry %i (%i available)...", n, list->count);
+	gp_log (GP_LOG_DEBUG, "gphoto2-port-info-list", _("Getting info of "
+		"entry %i (%i available)..."), n, list->count);
 
 	if (n < 0 || n >= list->count)
 		return (GP_ERROR_BAD_PARAMETERS);
