@@ -1,6 +1,6 @@
 /* gphoto2-port-version.c
  *
- * Copyright (C) 2002 Hans Ulrich Niedermann <gp@n-dimensional.de
+ * Copyright  2002 Hans Ulrich Niedermann <gp@n-dimensional.de
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include <stdlib.h>
 
 #include <gphoto2-port-version.h>
@@ -33,7 +33,7 @@ const char **gp_port_library_version(GPVersionVerbosity verbose)
 #ifdef HAVE_CC
 			HAVE_CC,
 #else
-			"unknown",
+			"unknown cc",
 #endif
 #ifdef HAVE_USB
 			"USB",
@@ -42,7 +42,10 @@ const char **gp_port_library_version(GPVersionVerbosity verbose)
 #endif
 #ifdef HAVE_SERIAL
 			"serial "
-#ifdef HAVE_BAUDBOY
+
+#ifdef HAVE_RESMGR
+			"resmgr locking",
+#elif HAVE_BAUDBOY
 			"baudboy locking",
 #elif HAVE_TTYLOCK
 			"ttylock locking",
@@ -51,6 +54,7 @@ const char **gp_port_library_version(GPVersionVerbosity verbose)
 #else
 			"without locking",
 #endif
+
 #else
 			"no serial",
 #endif
@@ -79,6 +83,11 @@ const char **gp_port_library_version(GPVersionVerbosity verbose)
 #else
 			"no serial (for serial cameras)",
 #endif
+#ifdef HAVE_RESMGR
+			"resmgr (serial port access and locking)",
+#else
+			"no resmgr (serial port access and locking)",
+#endif
 #ifdef HAVE_BAUDBOY
 			"baudboy (serial port locking)",
 #else
@@ -97,7 +106,7 @@ const char **gp_port_library_version(GPVersionVerbosity verbose)
 #ifdef HAVE_LTDL
 			"ltdl (hopefully with non-buggy libltdl :-)",
 #else
-			"no ltdl (working around bugg libltdl, eh? :-)",
+			"no ltdl (working around buggy libltdl, eh? :-)",
 #endif
 			NULL
 		};
