@@ -1,13 +1,13 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
- * Copyright (C) 2001 Lutz Mller <urc8@rz.uni-karlsruhe.de>
- * Copyright (C) 2000 Philippe Marzouk <pmarzouk@bigfoot.com>
- * Copyright (C) 2000 Edouard Lafargue <Edouard.Lafargue@bigfoot.com>
- * Copyright (C) 1999 Johannes Erdfelt <johannes@erdfelt.com>
- * Copyright (C) 1999 Scott Fritzinger <scottf@unr.edu>
+ * Copyright  2001 Lutz Mller <lutz@users.sf.net>
+ * Copyright  2000 Philippe Marzouk <pmarzouk@bigfoot.com>
+ * Copyright  2000 Edouard Lafargue <Edouard.Lafargue@bigfoot.com>
+ * Copyright  1999 Johannes Erdfelt <johannes@erdfelt.com>
+ * Copyright  1999 Scott Fritzinger <scottf@unr.edu>
  *
  * Based on work by:
- * Copyright (C) 1999 Beat Christen <spiff@longstreet.ch>
+ * Copyright  1999 Beat Christen <spiff@longstreet.ch>
  * 	for the toshiba gPhoto library.
  *                   
  *
@@ -27,7 +27,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include "config.h"
 #include <gphoto2-port-library.h>
 
 #include <stdlib.h>
@@ -61,6 +61,10 @@
 #  include <sgtty.h>
 #endif
 
+#ifdef HAVE_RESMGR
+#  include <resmgr.h>
+#endif
+
 #ifdef HAVE_BAUDBOY
 #  include <baudboy.h>
 #elif defined(HAVE_TTYLOCK)
@@ -208,6 +212,14 @@ gp_port_serial_lock (GPPort *dev, const char *path)
 	gp_log (GP_LOG_DEBUG, "gphoto2-port-serial",
 		"Trying to lock '%s'...", path);
 
+#ifdef HAVE_RESMGR
+	if (!rsm_lock_device(path)) {
+		return GP_OK;
+	}
+	/* fallthrough */
+#define __HAVE_LOCKING	
+#endif
+
 #if defined(HAVE_TTYLOCK) || defined(HAVE_BAUDBOY)
 	if (ttylock ((char*) path)) {
 		if (dev)
@@ -215,6 +227,7 @@ gp_port_serial_lock (GPPort *dev, const char *path)
 						  "'%s'"), path);
 		return (GP_ERROR_IO_LOCK);
 	}
+#define __HAVE_LOCKING	
 #elif defined(HAVE_LOCKDEV)
 	pid = dev_lock (path);
 	if (pid) {
@@ -229,12 +242,17 @@ gp_port_serial_lock (GPPort *dev, const char *path)
 		}
 		return (GP_ERROR_IO_LOCK);
 	}
-#else
-#warning No locking library found. 
-#warning You will run into problems if you use
-#warning gphoto2 with a serial (RS232) camera in 
-#warning combination with Konqueror (KDE) or Nautilus (GNOME).
-#warning This will *not* concern USB cameras.
+#define __HAVE_LOCKING	
+#endif
+
+#ifndef __HAVE_LOCKING
+# ifdef __GCC__
+#  warning No locking library found. 
+#  warning You will run into problems if you use
+#  warning gphoto2 with a serial (RS232) camera in 
+#  warning combination with Konqueror (KDE) or Nautilus (GNOME).
+#  warning This will *not* concern USB cameras.
+# endif
 #endif
 
 	return (GP_OK);
@@ -243,6 +261,12 @@ gp_port_serial_lock (GPPort *dev, const char *path)
 static int
 gp_port_serial_unlock (GPPort *dev, const char *path)
 {
+#ifdef HAVE_RESMGR
+	if (!rsm_unlock_device(path))
+		return GP_OK;
+	/* fallback through other unlock styles */
+#endif
+
 #if defined(HAVE_TTYLOCK) || defined(HAVE_BAUDBOY)
 	if (ttyunlock ((char*) path)) {
 		if (dev)
@@ -313,7 +337,16 @@ gp_port_library_list (GPPortInfoList *list)
 			continue;
 			
 		/* Device locked. Try to open the device. */
-		fd = open (path, O_RDONLY | O_NDELAY);
+		fd = -1;
+#ifdef HAVE_RESMGR
+		/* resmgr has its own API, which calls to a server and
+		 * communicates over UNIX domain sockets.
+		 */
+		fd = rsm_open_device(path, O_RDONLY | O_NDELAY);
+		/* fall through to standard open if this failed */
+#endif
+		if (fd == -1)
+			fd = open (path, O_RDONLY | O_NDELAY);
 		if (fd < 0) {
 			gp_port_serial_unlock (NULL, path);
 			continue;
@@ -403,6 +436,7 @@ gp_port_serial_open (GPPort *dev)
 		}
 		CHECK (result);
 	}
+	dev->pl->fd = -1;
 
 #if defined(__FreeBSD__) || defined(__APPLE__)
         dev->pl->fd = open (port, O_RDWR | O_NOCTTY | O_NONBLOCK);
@@ -411,7 +445,12 @@ gp_port_serial_open (GPPort *dev)
 	dev->pl->fd = open (port, O_RDWR | O_BINARY);
         close(fd);
 #else
-        dev->pl->fd = open (port, O_RDWR | O_NOCTTY | O_SYNC | O_NONBLOCK);
+# ifdef HAVE_RESMGR
+        dev->pl->fd = rsm_open_device (port, O_RDWR | O_NOCTTY | O_SYNC | O_NONBLOCK);
+	/* fall back trying old style open if not possible */
+# endif
+	if (dev->pl->fd == -1)
+		dev->pl->fd = open (port, O_RDWR | O_NOCTTY | O_SYNC | O_NONBLOCK);
 #endif
         if (dev->pl->fd == -1) {
 		gp_port_set_error (dev, _("Failed to open '%s' (%m)."), port);
@@ -541,7 +580,7 @@ gp_port_serial_read (GPPort *dev, char *bytes, int size)
 		    now = read (dev->pl->fd, bytes, 1);
 		    if (now < 0)
 			    return GP_ERROR_IO_READ;
-		    if (bytes[0] == -1) {
+		    if (bytes[0] == (char)-1) {
 			now = read (dev->pl->fd, bytes, 1);
 			if (now < 0)
 				return GP_ERROR_IO_READ;
@@ -560,7 +599,7 @@ gp_port_serial_read (GPPort *dev, char *bytes, int size)
 			    gp_port_set_error (dev, _("Parity error."));
 			    return GP_ERROR_IO_READ;
 			}
-			if (bytes[0] != -1) {
+			if (bytes[0] != (char)-1) {
 			    gp_port_set_error (dev, _("Unexpected parity response sequence 0xff 0x%02x."), bytes[0]);
 			    return GP_ERROR_IO_READ;
 			}
@@ -632,7 +671,9 @@ gp_port_serial_get_pin (GPPort *dev, GPPin pin, GPLevel *level)
         }
         *level = j & bit;
 #else
-#warning ACCESSING PINS IS NOT IMPLEMENTED FOR NON-TERMIOS SYSTEMS!
+# ifdef __GCC__
+#  warning ACCESSING PINS IS NOT IMPLEMENTED FOR NON-TERMIOS SYSTEMS!
+# endif
 #endif
 
         return (GP_OK);
@@ -664,7 +705,9 @@ gp_port_serial_set_pin (GPPort *dev, GPPin pin, GPLevel level)
 		return GP_ERROR_IO;
 	}
 #else
-#warning ACCESSING PINS IS NOT IMPLEMENTED FOR NON-TERMIOS SYSTEMS!
+# ifdef __GCC__
+#  warning ACCESSING PINS IS NOT IMPLEMENTED FOR NON-TERMIOS SYSTEMS!
+# endif
 #endif
 
         return GP_OK;
@@ -687,7 +730,9 @@ gp_port_serial_flush (GPPort *dev, int direction)
 		return (GP_ERROR_IO);
 	}
 #else
-#warning SERIAL FLUSH NOT IMPLEMENTED FOR NON TERMIOS SYSTEMS!
+# ifdef __GCC__
+#  warning SERIAL FLUSH NOT IMPLEMENTED FOR NON TERMIOS SYSTEMS!
+# endif
 #endif
 
 	return (GP_OK);
@@ -800,12 +845,10 @@ gp_port_serial_check_speed (GPPort *dev)
         tio.c_cflag = (tio.c_cflag & ~CSIZE) | CS8;
 
         /* Set into raw, no echo mode */
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__)
         tio.c_iflag &= ~(IGNBRK | IGNCR | INLCR | ICRNL |
                          IXANY | IXON | IXOFF | INPCK | ISTRIP);
-#else
-        tio.c_iflag &= ~(IGNBRK | IGNCR | INLCR | ICRNL | IUCLC |
-                         IXANY | IXON | IXOFF | INPCK | ISTRIP);
+#ifdef IUCLC
+        tio.c_iflag &= ~IUCLC;
 #endif
         tio.c_iflag |= (BRKINT | IGNPAR);
         tio.c_oflag &= ~OPOST;
@@ -843,15 +886,20 @@ gp_port_serial_check_speed (GPPort *dev)
 	 * Verify if the speed change has been successful.
 	 * This is needed because some Solaris systems just ignore unsupported
 	 * speeds (like 115200) instead of complaining.
+	 *
+	 * Only perform the check if we really changed to some speed.
 	 */
-	if (tcgetattr (dev->pl->fd, &tio)) {
-		GP_DEBUG ("Error on 'tcgetattr'.");
-		return (GP_ERROR_IO_SERIAL_SPEED);
-	}
-	if ((cfgetispeed (&tio) != speed) || (cfgetospeed (&tio) != speed)) {
-		GP_DEBUG ("Cannot set baudrate to %d...",
-			  dev->settings.serial.speed);
-		return (GP_ERROR_NOT_SUPPORTED);
+	if (speed != B0) {
+		if (tcgetattr (dev->pl->fd, &tio)) {
+			GP_DEBUG ("Error on 'tcgetattr'.");
+			return (GP_ERROR_IO_SERIAL_SPEED);
+		}
+		if ((cfgetispeed (&tio) != speed) ||
+		    (cfgetospeed (&tio) != speed)) {
+			GP_DEBUG ("Cannot set baudrate to %d...",
+				  dev->settings.serial.speed);
+			return (GP_ERROR_NOT_SUPPORTED);
+		}
 	}
 
 #else /* !HAVE_TERMIOS_H */
@@ -899,7 +947,9 @@ gp_port_serial_send_break (GPPort *dev, int duration)
         tcsendbreak (dev->pl->fd, duration / 310);
         tcdrain (dev->pl->fd);
 #else
-#warning SEND BREAK NOT IMPLEMENTED FOR NON TERMIOS SYSTEMS!
+# ifdef __GCC__
+#  warning SEND BREAK NOT IMPLEMENTED FOR NON TERMIOS SYSTEMS!
+# endif
 #endif
 
         return GP_OK;
