@@ -1,4 +1,4 @@
-/* $Id: print-usb-usermap.c,v 1.5 2003/01/10 23:23:59 marcusmeissner Exp $
+/* $Id: print-usb-usermap.c,v 1.7 2005/02/12 22:37:55 hun Exp $
  *
  * Copyright  2002 Hans Ulrich Niedermann <hun@users.sourceforge.net>
  * Portions Copyright  2002 Lutz Mller <lutz@users.sourceforge.net>
@@ -20,6 +20,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#define GP_USB_HOTPLUG_SCRIPT "usbcam"
+
 #define HELP_TEXT \
 "print-usb-usermap - print usb.usermap file for linux-hotplug\n" \
 "\n" \
@@ -34,7 +36,7 @@
 "program aborts regardless of data printed on stdout and returns a non-zero\n" \
 "status code.\n" \
 "If no <scriptname> is given, print-usb-usermap uses the script name\n" \
-"\"usbcam\"."
+"\"" GP_USB_HOTPLUG_SCRIPT "\"."
 
 #include <stdio.h>
 #include <sys/time.h>
@@ -51,8 +53,6 @@
 #define FALSE (0!=0)
 #endif
 
-#define GP_USB_HOTPLUG_SCRIPT "usbcam"
-
 #define GP_USB_HOTPLUG_MATCH_VENDOR_ID          0x0001
 #define GP_USB_HOTPLUG_MATCH_PRODUCT_ID         0x0002
 
@@ -60,7 +60,28 @@
 #define GP_USB_HOTPLUG_MATCH_DEV_SUBCLASS       0x0100
 #define GP_USB_HOTPLUG_MATCH_DEV_PROTOCOL       0x0200
 
-#define CR(result) {int r = (result); if (r < 0) return (r);}
+#ifdef __GNUC__
+#define CR(result) \
+	do { \
+		int r = (result); \
+		if (r < 0) { \
+			fprintf(stderr, "print-usb-usermap: " \
+				"Fatal error running `%s'.\n" \
+				"Aborting.\n", #result ); \
+			return (r); \
+		} \
+	} while (0)
+#else /* !__GNUC__ */
+#define CR(result) \
+	do { \
+		int r = (result); \
+		if (r < 0) { \
+			fprintf(stderr, "print-usb-usermap: " \
+				"Fatal error detected, aborting.\n"); \
+			return (r); \
+		} \
+	} while (0)
+#endif /* __GNUC__ */
 
 /* print-usb-usermap
  *
